﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using CMA.MovieRentalStore.EnterpriseModel;
using System.Collections.ObjectModel;
using CMA.MovieRentalStore.EnterpriseModel.Logging;

namespace CMA.MovieRentalStore.Presentation.Controls
{
    /// <summary>
    /// Codebehind für Rental control
    /// </summary>
    internal partial class RentalControl : UserControl
    {
        #region Constructor
        internal RentalControl()
        {
            InitializeComponent();

            Loaded += (sender, eventArgs) =>
            {
                LoadPersons();
                LoadMovies();
                LoadRentals();
            };
        }
        #endregion

        #region Fields
        private ObservableCollection<Rental> Rentals
        {
            set;
            get;
        }
        #endregion

        #region Methods
        private void LoadRentals()
        {
            try
            {
                Rentals = new ObservableCollection<Rental>(RentalService.Current.GetRepository<Rental>().GetAll());
                DataContext = Rentals;
            }
            catch (Exception exception)
            {
                ExceptionLogger.LogException(exception);
                MessageBox.Show("Ein Fehler ist aufgetreten");
            }
        }

        private void LoadMovies()
        {
            try
            {
                ComboBoxMovies.DataContext = RentalService.Current.GetRepository<DVD>().GetAll();
            }
            catch (Exception exception)
            {
                ExceptionLogger.LogException(exception);
                MessageBox.Show("Ein Fehler ist aufgetreten");
            }
        }

        private void LoadPersons()
        {
            try
            {
                ComboBoxPersons.DataContext = RentalService.Current.GetRepository<Person>().GetAll();
            }
            catch (Exception exception)
            {
                ExceptionLogger.LogException(exception);
                MessageBox.Show("Ein Fehler ist aufgetreten");
            }
        }

        private void ComboBoxPersons_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var selectedPerson = ComboBoxPersons.SelectedItem as string;

            if (selectedPerson == null)
                DataContext = Rentals;
            else
            {
                try
                {
                    var personContent = selectedPerson.Split(',');
                    DataContext = Rentals.Where(rental => rental.Person.Equals(RentalService.Current.GetRepository<Person>().GetAll(person => person.FirstName.Equals(personContent[1]) && person.LastName.Equals(personContent[0])).FirstOrDefault()));
                }
                catch (Exception exception)
                {
                    ExceptionLogger.LogException(exception);
                    MessageBox.Show("Ein Fehler ist aufgetreten");
                }
            }
        }

        private void ButtonMovieAdd_Click(object sender, RoutedEventArgs e)
        {
            var selectedTitle = ComboBoxMovies.SelectedItem as string;

            if(string.IsNullOrEmpty(selectedTitle))
                return;
            
            try
            {
                var selectedMovie = RentalService.Current.GetRepository<DVD>().GetAll(movie => movie.Title.Equals(selectedTitle)).FirstOrDefault();

                if (selectedMovie != null && !ListBoxMovies.Items.Contains(selectedMovie.Title))
                    ListBoxMovies.Items.Add(selectedMovie.Title);
            }
            catch (Exception exception)
            {
                ExceptionLogger.LogException(exception);
                MessageBox.Show("Ein Fehler ist aufgetreten");
            }
        }

        private void ButtonMovieRemove_Click(object sender, RoutedEventArgs e)
        {
            var selectedTitle = ListBoxMovies.SelectedItem as string;

            if (string.IsNullOrEmpty(selectedTitle))
                return;

            try
            {
                var selectedMovie = RentalService.Current.GetRepository<DVD>().GetAll(movie => movie.Title.Equals(selectedTitle)).FirstOrDefault();

                if (selectedMovie != null && ListBoxMovies.Items.Contains(selectedMovie.Title))
                    ListBoxMovies.Items.Remove(selectedMovie.Title);
            }
            catch (Exception exception)
            {
                ExceptionLogger.LogException(exception);
                MessageBox.Show("Ein Fehler ist aufgetreten");
            }
        }

        private void ButtonAdd_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                var rental = InitializeRental();

                //Prüfen auf Filme, deren Altersfreigabe höher als das Alter des Kunden ist
                foreach (var movie in rental.DVDs.Where(movie => movie.AgeRating > rental.Person.Age))
                {
                    MessageBox.Show(string.Format("Sie sind zu jung für den Film {0}.\r\nEntfernen Sie den Film aus der Liste.", movie.Title));
                    return;
                }

                if (!Rentals.Contains(rental))
                    Rentals.Add(rental);

                RentalService.Current.GetRepository<Rental>().Add(rental);

                ListBoxMovies.Items.Clear();
            }
            catch (Exception exception)
            {
                ExceptionLogger.LogException(exception);
                MessageBox.Show("Ein Fehler ist aufgetreten");
            }
        }

        private void ButtonUpdate_Click(object sender, RoutedEventArgs e)
        {
            var rental = ListViewRentals.SelectedItem as Rental;

            if (rental == null)
            {
                MessageBox.Show("Wählen Sie zuerst eine Ausleihe aus");
                return; 
            }

            var currentRental = Rentals.FirstOrDefault(aRental => aRental.ID.Equals(rental.ID));

            if (currentRental == null)
            {
                MessageBox.Show("Ein entsprechender Eintrag existiert nicht");
                return; 
            }

            currentRental.RentalStartDate = rental.RentalStartDate;
            currentRental.RentalEndDate = CheckBoxRentalEndDate.IsChecked == true ? DateTime.Now : new DateTime?();
            
            if ((ComboBoxRentalPersons.SelectedItem as Person) != null && !currentRental.Person.Equals((ComboBoxRentalPersons.SelectedItem as Person)))
                currentRental.Person = (ComboBoxRentalPersons.SelectedItem as Person);
            
            try
            {
                RentalService.Current.GetRepository<Rental>().Update(currentRental);

                ListBoxMovies.Items.Clear();
            }
            catch (Exception exception)
            {
                ExceptionLogger.LogException(exception);
                MessageBox.Show("Ein Fehler ist aufgetreten");
            }
        }

        private void ButtonDelete_Click(object sender, RoutedEventArgs e)
        {
            var rental = ListViewRentals.SelectedItem as Rental;

            if (rental == null)
            {
                MessageBox.Show("Wählen Sie zuerst eine Ausleihe aus");
                return; 
            }

            try
            {
                RentalService.Current.GetRepository<Rental>().Delete(rental.ID);

                Rentals.Remove(rental);

                ListBoxMovies.Items.Clear();
            }
            catch (Exception exception)
            {
                ExceptionLogger.LogException(exception);
                MessageBox.Show("Ein Fehler ist aufgetreten");
            }
        }

        private Rental InitializeRental()
        {
            var rental = new Rental();
            rental.RentalStartDate = DateTime.Now;

            var selectedPerson = ComboBoxRentalPersons.SelectedItem as string;

            var personContent = selectedPerson.Split(',');

            rental.Person = RentalService.Current.GetRepository<Person>().GetAll(person => person.FirstName.Equals(personContent[1]) && person.LastName.Equals(personContent[0])).FirstOrDefault(); 

            foreach (var title in ListBoxMovies.Items.OfType<string>()) //TODO
            {
                rental.DVDs.Add(RentalService.Current.GetRepository<DVD>().GetAll(movie => movie.Title.Equals(title)).FirstOrDefault());
            }

            return rental;
        }

        private void ListViewRentals_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var selectedRental = ListViewRentals.SelectedItem as Rental;

            if (selectedRental == null)
                return;

            ListBoxMovies.Items.Clear();

            foreach (var movie in selectedRental.DVDs)
                ListBoxMovies.Items.Add(movie.Title);

            CheckBoxRentalEndDate.IsChecked = selectedRental.RentalEndDate.HasValue;
        }
        #endregion
    }
}
